'use strict';
(function ($) {
    /**
     * Sanity Dialog
     * Version: 1.1.3 (29/01/2020)
     * https://checksanity.ru
     */
    $.fn.sanityDialog = function (options) {
        return new sanityDialogClass($(this), options);
    };

    class sanityDialogClass {
        constructor(dialog, options) {
            this.mVars = {
                timeout: 1,
                trigger: null,
                canClose: true
            };
            this.mElements = {
                dialog: null,
                close: null,
            };
            this.mWorking = false;
            $.extend(this.mVars, options);
            this.mElements.dialog = dialog;
            this.mElements.dialog.addClass('sanity_dialog');
            // If close button exist - bind it, else create it
            if ($('.sanity_dialog__close', this.mElements.dialog).length) {
                this.mElements.close = $('.sanity_dialog__close', this.mElements.dialog);
            } else {
                this.mElements.close = $('<button class="sanity_dialog__close"></button>').appendTo(this.mElements.dialog);
            }
            // If content in not wrapped, wrap it
            if (!$('.sanity_dialog__content', this.mElements.dialog).length) {
                dialog.wrapInner('<div class="sanity_dialog__content">');
            }
            // If trigger is not set from JS, set it from HTML (html is prefered method)
            if (!this.mVars.trigger) {
                this.mVars.trigger = this.mElements.dialog.attr('sanity_dialog');
            }
            // If link contains hash with dialog trigger - open it
            if (window.location.hash === '#dialog_' + this.mVars.trigger) {
                this.showDialog();
            }
            // Open dialog on trigger click
            $('[sanity_dialog-trigger="' + this.mVars.trigger + '"]').on('click', () => {
                this.openDialog();
            });
            // Open event for dialog
            $(this.mElements.dialog).on('open', (event, params) => {
                this.openDialog();
            });
            // Hide dialog if clicked something not close button or overlay
            $(this.mElements.dialog).on('click', (e) => {
                let target = $(e.target);
                if (target.is(this.mElements.dialog) || target.is(this.mElements.close) || target.is($('*', this.mElements.close))) {
                    this.closeDialog();
                }
            });
            // Close event for dialog
            $(this.mElements.dialog).on('close', (event, params) => {
                this.closeDialog();
            });

            // Binding popstate event
            $(window).bind('popstate', (e) => {
                // If popstate to something - hide dialog, else show it
                if (window.location.hash !== '#dialog_' + this.mVars.trigger) {
                    this.hideDialog();
                } else {
                    this.showDialog();
                }
            });
        }

        openDialog() {
            if (window.location.hash !== '#dialog_' + this.mVars.trigger) {
                window.location.hash = '#dialog_' + this.mVars.trigger;
            } else {
                this.showDialog();
            }
        }

        closeDialog() {
            if (window.location.hash === '#dialog_' + this.mVars.trigger) {
                history.back();
            } else {
                this.hideDialog();
            }
        }

        showDialog() {
            if (!this.mWorking && !this.mElements.dialog.hasClass('sanity_dialog_display')) {
                this.mWorking = true;
                this.mElements.dialog.addClass('sanity_dialog_display');
                $('html').addClass('overflow_hidden--sanity_dialog');
                this.mElements.dialog.removeClass('sanity_dialog_out').addClass('sanity_dialog_in');
                this.mElements.dialog.one('webkitAnimationEnd oanimationend msAnimationEnd animationend', () => {
                    this.mWorking = false;
                });
            }
        };

        hideDialog() {
            if (this.mVars.canClose) {
                if (!this.mWorking && this.mElements.dialog.hasClass('sanity_dialog_display')) {
                    this.mWorking = true;
                    this.mElements.dialog.removeClass('sanity_dialog_in').addClass('sanity_dialog_out');
                    this.mElements.dialog.one('webkitAnimationEnd oanimationend msAnimationEnd animationend', () => {
                        this.mWorking = false;
                        this.mElements.dialog.removeClass('sanity_dialog_display');
                        if (window.location.hash.includes('#dialog_')) {
                            // Do not hide overflow, because we still have dialog window
                        } else {
                            $('html').removeClass('overflow_hidden--sanity_dialog');
                        }
                    });
                }
            }
        };
    }
})(jQuery);